<?php
/// @file spw_chat_javarpc.php
/// @brief Chat Class for Javascript with AJAX technology\n
///        needs php as backend. \n
///
/// @verbatim
/// This file is called by java AJAX call from spw_chat.php.
/// 
/// @endverbatim
///
/// @author Leo Eibler
/// @date 20051217 le@sprossenwanne.at \n
///                created \n
/// @date 20051218 le@sprossenwanne.at \n
///                bugfix \n
/// @date 20070318 le@sprossenwanne.at \n
///                bugfix \n
/// @date 20070329 le@sprossenwanne.at \n
///                when calling hearall only send last x lines \n
///
/// This Software is for free use.
///
/// got from: http://www.eibler.at/
///
  define( 'SPW_CHAT_DIR', 'chat' );
  define( 'SPW_CHAT_USERSTATE', 'user.states' );
  define( 'SPW_CHAT_HISTORY', 'chat.history' );
  define( 'SPW_CHAT_INACTIVE', 3*60 ); 
  define( 'HEARALL_LINES', 20 );

// general functions from functions.php
  function my_xml_encode( $str ) {
      $str = str_replace( '&', '&amp;', $str );
      $str = str_replace( '<', '&lt;', $str );
      $str = str_replace( '>', '&gt;', $str );
      $str = str_replace( '\\"', '"', $str );
      $str = utf8_encode( $str );
      return $str;
  }
  
  function my_xml_decode( $str ) {
      $str = utf8_decode( $str );
      $str = str_replace( '\\\'', '\'', $str ) ;
      $str = str_replace( '&quot;', '"', $str ) ;
      $str = str_replace( '&lt;', "<", $str );
      $str = str_replace( '&gt;', ">", $str );
      return my_display_decode( $str ); // patch 20051102 by leo
  }

  function my_display_decode( $str ) {
      $str = str_replace( '"', '&quot;', $str ) ;
      $str = str_replace( '<', "&lt;", $str );
      $str = str_replace( '>', "&gt;", $str );
      return $str;
  }

// special chat functions
  function saveUserState( $user = false ) {
      global $spwChatUserState;
      if( is_array( $spwChatUserState ) ) {
         foreach( $spwChatUserState as $k=>$m ) {
             if( ($m['date']+SPW_CHAT_INACTIVE) < mktime() ) {
                 // user is 10 minutes inactive ... delete
                 sendMsg2All( '<span class="italic"><span class="fett">'.$k.'</span> verl&auml;&szlig;t den Chat</span>.');
                 @unlink( SPW_CHAT_DIR.'/'.$k.'.txt' );
                 unset( $spwChatUserState[$k] );
             }
         }
         if( $user !== false ) {
             if( !isset( $spwChatUserState[$user] ) ) {
                 sendMsg2All( '<span class="italic"><span class="fett">'.$user.'</span> betritt den Chat</span>.');
                 $spwChatUserState[$user] = array( 'login'=>mktime(),
                                                   'date'=>mktime(),
                                                   'active'=>mktime(),
                                                   'color'=>sprintf('%02X%02X%02X', (mt_rand()%16)*16, (mt_rand()%16)*16, (mt_rand()%16)*16) 
                                                 );
             } else {
                 $spwChatUserState[$user]['date'] = mktime();
             }
         }
      }
      @file_put_contents( SPW_CHAT_DIR.'/'.SPW_CHAT_USERSTATE, serialize( $spwChatUserState ) );
  }

  function formatUser( $user ) {
      global $spwChatUserState;
      if( isset( $spwChatUserState[$user]['color'] ) ) {
          return '<span style="color:#'.$spwChatUserState[$user]['color'].';">'.$user.'</span>&gt; ';
      } else {
          return $user.'&gt; ';
      }
  }
  
  function loadUserState( ) {
      global $spwChatUserState;
      $spwChatUserState = array();
      $content = @file_get_contents( SPW_CHAT_DIR.'/'.SPW_CHAT_USERSTATE );
      $spwChatUserState = unserialize( $content );
      if( !is_array( $spwChatUserState ) ) {
          $spwChatUserState = array();
      }
  }

  function sendMsg2User( $user, $text ) {
      $content = @file_get_contents( SPW_CHAT_DIR.'/'.$user.'.txt' );
      @file_put_contents( SPW_CHAT_DIR.'/'.$user.'.txt', $text."\n".$content );
  }
  
  function getMsg4User( $user ) {
      $out = '';
      if( file_exists( SPW_CHAT_DIR.'/'.$user.'.txt' ) ) {
          // get user chat file
          $out = @file_get_contents( SPW_CHAT_DIR.'/'.$user.'.txt' );
      }
      @file_put_contents( SPW_CHAT_DIR.'/'.$user.'.txt', '' );
      return $out;
  }
  
  function sendMsg2All( $text ) {
      global $spwChatUserState;

      // save message to chat history file
      $content = @file_get_contents( SPW_CHAT_DIR.'/'.SPW_CHAT_HISTORY );
      @file_put_contents( SPW_CHAT_DIR.'/'.SPW_CHAT_HISTORY, $text."\n".$content );
      
      // send message to each user
      if( is_array( $spwChatUserState ) ) {
          foreach( $spwChatUserState as $k=>$m ) {
             sendMsg2User( $k, $text );
          }
      }
  }
  
  function SendErrorMsg2User( $user, $text ) {
      sendMsg2User( $user, '<span class="italic"><span class="fett">SYSTEM</span>&gt; ERROR: '.$text.'</span>' );
  }
  
  function getMsg4All( ) {
      $fp = @fopen( SPW_CHAT_DIR.'/'.SPW_CHAT_HISTORY, 'rb' );
      $out = '';
      if( $fp ) {
          $i = 0;
          while( !feof($fp) && ($i < HEARALL_LINES) ) {
              $out .= @fgets( $fp ); 
              $i++;
          }
      }
      //$out = @file_get_contents( SPW_CHAT_DIR.'/'.SPW_CHAT_HISTORY );
      return $out;
  }

  function spw_chat_helpcommands() {
      $out = '<table>'.
             '<tr><th>Kommando</th><th>Beschreibung</th></tr>'.
             '<tr> <td>/h</td> <td>diese Hilfe</td> </tr>'.
             '<tr> <td>/r text</td> <td>Stellt den Text in Regenbogenfarben dar ... also BUNT</td> </tr>'.
             '<tr> <td>/f user text</td> <td>Flstert den Text an User</td> </tr>'.
             '<tr> <td>/c</td> <td>Eigene Farbe ndern</td> </tr>'.
             '<tr> <td>/l</td> <td>Anzeige aller eingeloggten User</td> </tr>'.
             '</table>';
      return $out;
  }
  
  
  $spwChatUserState = array();
  loadUserState( );

  mt_srand(microtime()*1324521);

  // we are a text/xml content
  header( 'Content-Type: text/xml');
  // for IE: do not cache our xml file
  header( 'Cache-Control: no-cache' );

  echo '<?xml version="1.0" encoding="UTF-8"?>';
  echo '<dataroot>';
  
  $out='';
  // start output buffering ... we don't want to corrupt our xml file
  ob_start();
  
  if( isset($_GET['hearall']) ) {
      $out = getMsg4All( );
  } // if( isset($_GET['hearall']) ) 
  else
  if( isset($_GET['shout']) && !empty($_POST['textme']) ) {
      $spwChatUserState[$_GET['user']]['active'] = mktime();
      $chatCommand = strtok( $_POST['textme'], ' ' );
      if( $chatCommand == '/h' ) {
          sendMsg2User( $_GET['user'], spw_chat_helpcommands() );
      }
      else
      if( $chatCommand == '/l' ) {
          $temp = '';
          if( is_array( $spwChatUserState ) ) {
              $count = 0;
              $temp .= 'User im Chat:<br />'.
                       '<table width="50%">'.
                       '<tr>'.
                       '<th width="25%">User</th> <th width="35%">Online seit</th>'.
                       '<th width="25%">inaktiv</th>'.
                       '</tr>';
              foreach( $spwChatUserState as $k=>$m ) {
                  $temp .= '<tr>'.
                           '<td>'.$k.'</td>'.
                           '<td>'.date( 'd.m.Y, H:i', $m['login'] ).' - '.
                           (int)(($m['date']-$m['login'])/60).' min</td>'.
                           '<td>'.(int)((mktime()-$m['active'])/60).' min</td>'.
                           '</tr>';
                  $count++;
              }
              $temp.= '</table>';
          }
          $temp .= $count.' User';
          sendMsg2User( $_GET['user'], $temp );
      } // if( $_POST['textme'] == '/l' )
      else 
      if( $chatCommand == '/f' ) {
          $f2user = strtok( ' ' );
          $temp = my_xml_encode( strtok( '' ) );
          if( !empty( $f2user ) && !empty( $temp ) ) {
              if( isset( $spwChatUserState[$f2user] ) ) {
                  sendMsg2User( $f2user, '<span class="italic"><span class="fett">'.formatUser( $_GET['user'] ).' fl&uuml;stert</span></span>: '.$temp );
                  sendMsg2User( $_GET['user'], '<span class="italic">'.$_GET['user'].'--&gt; <span class="fett">'.$f2user.'</span></span>: '.$temp );
              } else {
                  SendErrorMsg2User( $_GET['user'], 'User <span class="fett">'.$f2user.'</span> gibts nicht.' );
              }
          } else {
              SendErrorMsg2User( $_GET['user'], '/f username nachricht.' );
          }
      } // if( strpos( $_POST['textme'], '/f ') === 0 )
      else 
      if( $chatCommand == '/c' ) {
          if( isset( $spwChatUserState[$_GET['user']] ) ) {
              $spwChatUserState[$_GET['user']]['color'] = sprintf('%02X%02X%02X', (mt_rand()%16)*16, (mt_rand()%16)*16, (mt_rand()%16)*16);
              sendMsg2All( $_GET['user'].' hat die <span style="color:#'.$spwChatUserState[$_GET['user']]['color'].'">Farbe</span> gewechselt.' );
          }
      } // if( $chatCommand == '/c' )
      else
      if( $chatCommand == '/r' ) {
          $temp = '';
          $t = substr( $_POST['textme'], 3 );
          if( strlen($t) > 0 ) {
              $incr = (int)(255/strlen($t));
              $color = 0;
              $colorrand = mt_rand()%4;
              $colorrand2 = (mt_rand()%16)*16;
              for( $i=0; $i<strlen($t); $i++ ) {
                   if( $colorrand == 0) {
                       $temp .= '<span style="color:#'.sprintf("%02X%02X%02X", $color, 255-$color, $colorrand2 ).';">'.$_POST['textme']{3+$i}.'</span>';
                   } else
                   if( $colorrand == 1) {
                       $temp .= '<span style="color:#'.sprintf("%02X%02X%02X", 255-$color, $color, $colorrand2 ).';">'.$_POST['textme']{3+$i}.'</span>';
                   } else
                   if( $colorrand == 2) {
                       $temp .= '<span style="color:#'.sprintf("%02X%02X%02X", 255-$color, $colorrand2, $color ).';">'.$_POST['textme']{3+$i}.'</span>';
                   } else
                   if( $colorrand == 3) {
                       $temp .= '<span style="color:#'.sprintf("%02X%02X%02X", $colorrand2, 255-$color, $color ).';">'.$_POST['textme']{3+$i}.'</span>';
                   }
                   $color += $incr;
              }
          } 
          sendMsg2All( formatUser( $_GET['user'] ).$temp );
      } // if( strpos( $_POST['textme'], '/t ') === 0 )
      else
      {
          $_POST['textme'] = my_xml_encode( $_POST['textme'] );
          sendMsg2All( formatUser( $_GET['user'] ).$_POST['textme'] );
      }
  } // if( isset($_GET['shout']) && !empty($_POST['textme']) )

  // check for message for this user
  $out = getMsg4User( $_GET['user'] ).$out;
  
  // delete error message
  $k =ob_get_contents();
  ob_end_clean();

  $k = '<![CDATA['.$out.']]>';

  echo my_xml_encode( $k );
  
  echo '</dataroot>';

  saveUserState( $_GET['user'] );

?>
